@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import inspect
 import math
@@ -75,6 +70,7 @@ RANDOM_METHODS = [
     for name in [
         "_randbelow",
         "betavariate",
+        "binomialvariate",
         "choice",
         "choices",
         "expovariate",
@@ -138,9 +134,9 @@ def define_copy_method(name):
         self._hypothesis_log_random(name, kwargs, result)
         return result
 
-    spec = inspect.getfullargspec(STUBS.get(name, target))
+    sig = inspect.signature(STUBS.get(name, target))
 
-    result = define_function_signature(target.__name__, target.__doc__, spec)(
+    result = define_function_signature(target.__name__, target.__doc__, sig)(
         implementation
     )
 
@@ -187,10 +183,10 @@ def normalize_zero(f: float) -> float:
 
 
 class ArtificialRandom(HypothesisRandom):
-    VERSION = 10 ** 6
+    VERSION = 10**6
 
     def __init__(self, note_method_calls, data):
-        HypothesisRandom.__init__(self, note_method_calls=note_method_calls)
+        super().__init__(note_method_calls=note_method_calls)
         self.__data = data
         self.__state = RandomState()
 
@@ -276,6 +272,9 @@ class ArtificialRandom(HypothesisRandom):
                 result = cu.integer_range(self.__data, start, stop - 1)
         elif method == "randint":
             result = cu.integer_range(self.__data, kwargs["a"], kwargs["b"])
+        # New in Python 3.12, so not taken by our coverage job
+        elif method == "binomialvariate":  # pragma: no cover
+            result = cu.integer_range(self.__data, 0, kwargs["n"])
         elif method == "choice":
             seq = kwargs["seq"]
             result = cu.integer_range(self.__data, 0, len(seq) - 1)
@@ -322,8 +321,7 @@ class ArtificialRandom(HypothesisRandom):
             result = self.__data.draw(floats(min_value=0.0))
         elif method == "shuffle":
             result = self.__data.draw(permutations(range(len(kwargs["x"]))))
-        # This is tested for but only appears in 3.9 so doesn't appear in coverage.
-        elif method == "randbytes":  # pragma: no cover
+        elif method == "randbytes":
             n = kwargs["n"]
             result = self.__data.draw(binary(min_size=n, max_size=n))
         else:
@@ -390,14 +388,14 @@ def convert_kwargs(name, kwargs):
         if args[-1] is signature.parameters[name].default:
             args.pop()
         else:
-            break  # pragma: no cover  # Only on Python < 3.8
+            break
 
     return (args, kwargs)
 
 
 class TrueRandom(HypothesisRandom):
     def __init__(self, seed, note_method_calls):
-        HypothesisRandom.__init__(self, note_method_calls)
+        super().__init__(note_method_calls)
         self.__seed = seed
         self.__random = Random(seed)
 
