@@ -10,6 +10,7 @@
 
 import builtins
 import collections
+import collections.abc
 import datetime
 import decimal
 import fractions
@@ -19,21 +20,18 @@ import io
 import ipaddress
 import numbers
 import os
+import random
 import re
 import sys
 import typing
 import uuid
 from pathlib import PurePath
 from types import FunctionType
+from typing import get_args, get_origin
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import (
-    PYPY,
-    BaseExceptionGroup,
-    ExceptionGroup,
-    get_origin,
-)
+from hypothesis.internal.compat import PYPY, BaseExceptionGroup, ExceptionGroup
 from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
 from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
@@ -82,11 +80,7 @@ try:
 except AttributeError:  # pragma: no cover
     pass  # `typing_extensions` might not be installed
 
-FinalTypes: tuple = ()
-try:
-    FinalTypes += (typing.Final,)
-except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+FinalTypes: tuple = (typing.Final,)
 try:
     FinalTypes += (typing_extensions.Final,)
 except AttributeError:  # pragma: no cover
@@ -96,7 +90,7 @@ ConcatenateTypes: tuple = ()
 try:
     ConcatenateTypes += (typing.Concatenate,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     ConcatenateTypes += (typing_extensions.Concatenate,)
 except AttributeError:  # pragma: no cover
@@ -106,7 +100,7 @@ ParamSpecTypes: tuple = ()
 try:
     ParamSpecTypes += (typing.ParamSpec,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     ParamSpecTypes += (typing_extensions.ParamSpec,)
 except AttributeError:  # pragma: no cover
@@ -116,7 +110,7 @@ TypeGuardTypes: tuple = ()
 try:
     TypeGuardTypes += (typing.TypeGuard,)
 except AttributeError:  # pragma: no cover
-    pass  # Is missing for `python<3.8`
+    pass  # Is missing for `python<3.10`
 try:
     TypeGuardTypes += (typing_extensions.TypeGuard,)
 except AttributeError:  # pragma: no cover
@@ -151,8 +145,7 @@ typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 # We use this to disallow all non-runtime types from being registered and resolved.
 # By "non-runtime" we mean: types that do not really exist in python's
 # and are just added for more fancy type annotations.
-# `Final` is a great example: it just indicates
-# that this value can't be reassigned.
+# `Final` is a great example: it just indicates that this value can't be reassigned.
 NON_RUNTIME_TYPES = (
     typing.Any,
     *ClassVarTypes,
@@ -185,15 +178,15 @@ for name in (
 
 def type_sorting_key(t):
     """Minimise to None, then non-container types, then container types."""
-    if not (is_a_type(t) or is_typing_literal(t)):  # This branch is for Python < 3.8
-        raise InvalidArgument(f"thing={t} must be a type")  # pragma: no cover
     if t is None or t is type(None):  # noqa: E721
         return (-1, repr(t))
-    t = getattr(t, "__origin__", t)
-    if not isinstance(t, type):  # pragma: no cover
-        # Some generics in the typing module are not actually types in 3.7
-        return (2, repr(t))
-    return (int(issubclass(t, collections.abc.Container)), repr(t))
+    t = get_origin(t) or t
+    try:
+        is_container = int(issubclass(t, collections.abc.Container))
+    except Exception:  # pragma: no cover
+        # e.g. `typing_extensions.Literal` is not a container
+        is_container = 0
+    return (is_container, repr(t))
 
 
 def _compatible_args(args, superclass_args):
@@ -214,11 +207,8 @@ def try_issubclass(thing, superclass):
     try:
         # In this case we're looking at two distinct classes - which might be generics.
         # That brings in some complications:
-        if issubclass(
-            getattr(thing, "__origin__", None) or thing,
-            getattr(superclass, "__origin__", None) or superclass,
-        ):
-            superclass_args = getattr(superclass, "__args__", None)
+        if issubclass(get_origin(thing) or thing, get_origin(superclass) or superclass):
+            superclass_args = get_args(superclass)
             if not superclass_args:
                 # The superclass is not generic, so we're definitely a subclass.
                 return True
@@ -250,15 +240,12 @@ def is_a_new_type(thing):
         )
     # In 3.10 and later, NewType is actually a class - which simplifies things.
     # See https://bugs.python.org/issue44353 for links to the various patches.
-    return isinstance(thing, typing.NewType)  # pragma: no cover  # on 3.8, anyway
+    return isinstance(thing, typing.NewType)
 
 
 def is_a_union(thing):
     """Return True if thing is a typing.Union or types.UnionType (in py310)."""
-    return (
-        isinstance(thing, UnionType)
-        or getattr(thing, "__origin__", None) is typing.Union
-    )
+    return isinstance(thing, UnionType) or get_origin(thing) is typing.Union
 
 
 def is_a_type(thing):
@@ -267,11 +254,9 @@ def is_a_type(thing):
 
 
 def is_typing_literal(thing):
-    return (
-        hasattr(typing, "Literal")
-        and getattr(thing, "__origin__", None) == typing.Literal
-        or hasattr(typing_extensions, "Literal")
-        and getattr(thing, "__origin__", None) == typing_extensions.Literal
+    return get_origin(thing) in (
+        typing.Literal,
+        getattr(typing_extensions, "Literal", object()),
     )
 
 
@@ -315,7 +300,8 @@ def is_generic_type(type_):
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
     # while the second condition is for `MyClass`.
     return isinstance(type_, typing_root_type + (GenericAlias,)) or (
-        isinstance(type_, type) and typing.Generic in type_.__mro__
+        isinstance(type_, type)
+        and (typing.Generic in type_.__mro__ or hasattr(type_, "__class_getitem__"))
     )
 
 
@@ -345,7 +331,7 @@ def from_typing_type(thing):
     # information to sensibly resolve to strategies at runtime.
     # Finally, we run a variation of the subclass lookup in `st.from_type`
     # among generic types in the lookup.
-    if getattr(thing, "__origin__", None) == tuple or isinstance(
+    if get_origin(thing) == tuple or isinstance(
         thing, getattr(typing, "TupleMeta", ())
     ):
         elem_types = getattr(thing, "__tuple_params__", None) or ()
@@ -359,7 +345,7 @@ def from_typing_type(thing):
         elif len(elem_types) == 1 and elem_types[0] == ():
             return st.tuples()  # Empty tuple; see issue #1583
         return st.tuples(*map(st.from_type, elem_types))
-    if hasattr(typing, "Final") and getattr(thing, "__origin__", None) == typing.Final:
+    if get_origin(thing) == typing.Final:
         return st.one_of([st.from_type(t) for t in thing.__args__])
     if is_typing_literal(thing):
         args_dfs_stack = list(thing.__args__)
@@ -388,8 +374,8 @@ def from_typing_type(thing):
         raise ResolutionFailed(f"Cannot resolve {thing} to a strategy")
 
     # Some "generic" classes are not generic *in* anything - for example both
-    # Hashable and Sized have `__args__ == ()` on Python 3.7 or later.
-    origin = getattr(thing, "__origin__", thing)
+    # Hashable and Sized have `__args__ == ()`
+    origin = get_origin(thing) or thing
     if (
         origin in vars(collections.abc).values()
         and len(getattr(thing, "__args__", None) or []) == 0
@@ -399,19 +385,28 @@ def from_typing_type(thing):
     # Parametrised generic types have their __origin__ attribute set to the
     # un-parametrised version, which we need to use in the subclass checks.
     # e.g.:     typing.List[int].__origin__ == typing.List
+    # (actually not sure if this is true since Python 3.9 or so)
     mapping = {
         k: v
         for k, v in _global_type_lookup.items()
         if is_generic_type(k) and try_issubclass(k, thing)
     }
-    if typing.Dict in mapping or typing.Set in mapping:
+    # Drop some unusual cases for simplicity
+    for weird in (tuple, getattr(os, "_Environ", None)):
+        if len(mapping) > 1:
+            mapping.pop(weird, None)
+    # After we drop Python 3.8 and can rely on having generic builtin types, we'll
+    # be able to simplify this logic by dropping the typing-module handling.
+    if {dict, set, typing.Dict, typing.Set}.intersection(mapping):
         # ItemsView can cause test_lookup.py::test_specialised_collection_types
         # to fail, due to weird isinstance behaviour around the elements.
+        mapping.pop(collections.abc.ItemsView, None)
         mapping.pop(typing.ItemsView, None)
-    if typing.Deque in mapping and len(mapping) > 1:
+    if {collections.deque, typing.Deque}.intersection(mapping) and len(mapping) > 1:
         # Resolving generic sequences to include a deque is more trouble for e.g.
         # the ghostwriter than it's worth, via undefined names in the repr.
-        mapping.pop(typing.Deque)
+        mapping.pop(collections.deque, None)
+        mapping.pop(typing.Deque, None)
     if len(mapping) > 1:
         # issubclass treats bytestring as a kind of sequence, which it is,
         # but treating it as such breaks everything else when it is presumed
@@ -419,7 +414,7 @@ def from_typing_type(thing):
         # Except for sequences of integers, or unions which include integer!
         # See https://github.com/HypothesisWorks/hypothesis/issues/2257
         #
-        # This block drops ByteString from the types that can be generated
+        # This block drops bytes from the types that can be generated
         # if there is more than one allowed type, and the element type is
         # not either `int` or a Union with `int` as one of its elements.
         elem_type = (getattr(thing, "__args__", None) or ["not int"])[0]
@@ -431,6 +426,8 @@ def from_typing_type(thing):
             isinstance(T, type) and issubclass(int, get_origin(T) or T)
             for T in list(union_elems) + [elem_type]
         ):
+            mapping.pop(bytes, None)
+            mapping.pop(collections.abc.ByteString, None)
             mapping.pop(typing.ByteString, None)
     elif (
         (not mapping)
@@ -438,9 +435,17 @@ def from_typing_type(thing):
         and thing.__forward_arg__ in vars(builtins)
     ):
         return st.from_type(getattr(builtins, thing.__forward_arg__))
+    # Before Python 3.9, we sometimes have e.g. Sequence from both the typing
+    # module, and collections.abc module.  Discard any type which is not it's own
+    # origin, where the origin is also in the mapping.
+    for t in sorted(mapping, key=type_sorting_key):
+        origin = get_origin(t)
+        if origin is not t and origin in mapping:
+            mapping.pop(t)
+    # Sort strategies according to our type-sorting heuristic for stable output
     strategies = [
         v if isinstance(v, st.SearchStrategy) else v(thing)
-        for k, v in mapping.items()
+        for k, v in sorted(mapping.items(), key=lambda kv: type_sorting_key(kv[0]))
         if sum(try_issubclass(k, T) for T in mapping) == 1
     ]
     empty = ", ".join(repr(s) for s in strategies if s.is_empty)
@@ -571,11 +576,13 @@ _global_type_lookup: typing.Dict[
     filter: st.builds(filter, st.just(lambda _: None), st.just(())),
     map: st.builds(map, st.just(lambda _: None), st.just(())),
     reversed: st.builds(reversed, st.just(())),
+    property: st.builds(property, st.just(lambda _: None)),
     classmethod: st.builds(classmethod, st.just(lambda self: self)),
     staticmethod: st.builds(staticmethod, st.just(lambda self: self)),
     super: st.builds(super, st.from_type(type)),
     re.Match: st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool),
     re.Pattern: st.builds(re.compile, st.sampled_from(["", b""])),
+    random.Random: st.randoms(),
     # Pull requests with more types welcome!
 }
 if zoneinfo is not None:  # pragma: no branch
@@ -593,24 +600,13 @@ if sys.version_info[:2] >= (3, 9):
     _global_type_lookup[os._Environ] = st.just(os.environ)
 
 
-try:  # pragma: no cover
-    import numpy as np
-
-    from hypothesis.extra.numpy import array_dtypes, array_shapes, arrays, scalar_dtypes
-
-    _global_type_lookup[np.dtype] = array_dtypes()
-    _global_type_lookup[np.ndarray] = arrays(scalar_dtypes(), array_shapes(max_dims=2))
-except ImportError:
-    pass
-
-
 _global_type_lookup.update(
     {
         # Note: while ByteString notionally also represents the bytearray and
         # memoryview types, it is a subclass of Hashable and those types are not.
-        # We therefore only generate the bytes type.
-        typing.ByteString: st.binary(),
-        collections.abc.ByteString: st.binary(),
+        # We therefore only generate the bytes type. type-ignored due to deprecation.
+        typing.ByteString: st.binary(),  # type: ignore
+        collections.abc.ByteString: st.binary(),  # type: ignore
         # TODO: SupportsAbs and SupportsRound should be covariant, ie have functions.
         typing.SupportsAbs: st.one_of(
             st.booleans(),
@@ -651,6 +647,7 @@ _global_type_lookup.update(
             # this generates strings that should able to be parsed into integers
             st.from_regex(r"\A-?\d+\Z").filter(functools.partial(can_cast, int)),
         ),
+        typing.SupportsIndex: st.integers() | st.booleans(),
         typing.SupportsBytes: st.one_of(
             st.booleans(),
             st.binary(),
@@ -662,8 +659,26 @@ _global_type_lookup.update(
         typing.TextIO: st.builds(io.StringIO, st.text()),
     }
 )
-if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
-    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()
+
+
+# The "extra" lookups define a callable that either resolves to a strategy for
+# this narrowly extra-specific type, or returns None to proceed with normal
+# type resolution. The callable will only be called if the module is
+# installed. To avoid the performance hit of importing anything here, we defer
+# it until the method is called the first time, at which point we replace the
+# entry in the lookup table with the direct call.
+def _from_numpy_type(thing: typing.Type) -> typing.Optional[st.SearchStrategy]:
+    from hypothesis.extra.numpy import _from_type
+
+    _global_extra_lookup["numpy"] = _from_type
+    return _from_type(thing)
+
+
+_global_extra_lookup: typing.Dict[
+    str, typing.Callable[[typing.Type], typing.Optional[st.SearchStrategy]]
+] = {
+    "numpy": _from_numpy_type,
+}
 
 
 def register(type_, fallback=None, *, module=typing):
@@ -675,17 +690,23 @@ def register(type_, fallback=None, *, module=typing):
             return lambda f: f
 
     def inner(func):
+        nonlocal type_
         if fallback is None:
             _global_type_lookup[type_] = func
             return func
 
         @functools.wraps(func)
         def really_inner(thing):
-            # This branch is for Python < 3.8, when __args__ was not always tracked
+            # This branch is for Python <= 3.8, when __args__ was not always tracked
             if getattr(thing, "__args__", None) is None:
                 return fallback  # pragma: no cover
             return func(thing)
 
+        if sys.version_info[:2] >= (3, 9):
+            try:
+                type_ = get_origin(type_)
+            except Exception:
+                pass
         _global_type_lookup[type_] = really_inner
         return really_inner
 
@@ -696,7 +717,7 @@ def register(type_, fallback=None, *, module=typing):
 @register("Type", module=typing_extensions)
 def resolve_Type(thing):
     if getattr(thing, "__args__", None) is None:
-        # This branch is for Python < 3.8, when __args__ was not always tracked
+        # This branch is for Python <= 3.8, when __args__ was not always tracked
         return st.just(type)  # pragma: no cover
     args = (thing.__args__[0],)
     if is_a_union(args[0]):
@@ -745,6 +766,7 @@ def _from_hashable_type(type_):
 
 
 @register(typing.Set, st.builds(set))
+@register(typing.MutableSet, st.builds(set))
 def resolve_Set(thing):
     return st.sets(_from_hashable_type(thing.__args__[0]))
 
@@ -811,9 +833,8 @@ def resolve_ChainMap(thing):
     return resolve_Dict(thing).map(collections.ChainMap)
 
 
-@register("OrderedDict", st.builds(dict).map(collections.OrderedDict))
+@register(typing.OrderedDict, st.builds(dict).map(collections.OrderedDict))
 def resolve_OrderedDict(thing):
-    # typing.OrderedDict is new in Python 3.7.2
     return resolve_Dict(thing).map(collections.OrderedDict)
 
 
@@ -887,7 +908,7 @@ def resolve_Callable(thing):
                 f"Callable type parametrized by {arg!r}.  Consider using an "
                 "explicit strategy, or opening an issue."
             )
-    if getattr(return_type, "__origin__", None) in TypeGuardTypes:
+    if get_origin(return_type) in TypeGuardTypes:
         raise InvalidArgument(
             "Hypothesis cannot yet construct a strategy for callables which "
             f"are PEP-647 TypeGuards (got {return_type!r}).  "
